@@ -4,7 +4,9 @@
 #'     DO NOT REMOVE.
 #' @import shiny
 #' @importFrom rlang .data
+#'
 #' @noRd
+
 app_server <- function(input, output, session) {
 
   # Load reference libraries
@@ -32,4 +34,11 @@ app_server <- function(input, output, session) {
     resources = resources
   )
 
+  # Covariate module
+  covariates_server(
+    session = session,
+    input = input,
+    output = output,
+    resources = resources
+  )
 }
